from pyVmomi import vim, vmodl


def wait_for_tasks(si, tasks):
    property_collector = si.content.propertyCollector
    task_list = [str(task) for task in tasks]

    obj_specs = [vmodl.query.PropertyCollector.ObjectSpec(obj=task)
                 for task in tasks]

    property_spec = vmodl.query.PropertyCollector.PropertySpec(
        type=vim.Task,
        pathSet=[],
        all=True
    )

    filter_spec = vmodl.query.PropertyCollector.FilterSpec()
    filter_spec.objectSet = obj_specs
    filter_spec.propSet = [property_spec]
    pcfilter = property_collector.CreateFilter(filter_spec, True)

    try:
        version, state = None, None

        while len(task_list):
            update = property_collector.WaitForUpdates(version)

            for filter_set in update.filterSet:
                for obj_set in filter_set.objectSet:
                    task = obj_set.obj

                    for change in obj_set.changeSet:

                        if change.name == 'info':
                            state = change.val.state

                        elif change.name == 'info.state':
                            state = change.val

                        else:
                            continue

                        if not str(task) in task_list:
                            continue

                        if state == vim.TaskInfo.State.success:
                            task_list.remove(str(task))

                        elif state == vim.TaskInfo.State.error:
                            raise task.info.error

            version = update.version

    finally:
        if pcfilter:
            pcfilter.Destroy()
